function varargout=plotmesh(self,varargin)
% FUNCTION plotmesh 
%   Plots the siMesh object.
%
% USAGE
%   Th.plotmesh() or plotmesh(Th) where Th is a siMesh object.
%
%   Th.plotmesh(key,value,...) allows additional key/value pairs to be 
%     used when displaying the siMesh object. The key strings could be
%
%     'd'      to specify the d-simplicial meshes. Default d=Th.d
%     'labels' to specify the labels of the elementary meshes. 
%              Default [] i.e all.
%     'legend' if true display the labels of the elementary meshes in
%              the legend box. Default false.
%     'color'  to specify the color of all the displayed elementary meshes.
%
%   or any of the plotmesh options depending of Th.dim and d. Same key/pairs as
%
%     - patch   function for dim=3 and d=3 (no face)
%     - trimesh function for dim=3 and d=2
%     - plot3   function for dim=3 and d=1
%     - plot3   function for dim=3 and d=0
%     - triplot function for dim=2 and d=2 
%     - plot    function for dim=2 and d=1
%     - plot    function for dim=2 and d=0 
%     - line    function for dim=1 and d=1
%     - plot    function for dim=1 and d=0
%
% SAMPLES
%   Th=fc_simesh.HyperCube(3,10);
%   figure(1)
%   Th.plotmesh()
%   figure(2)
%   Th.plotmesh('d',2,'labels',[1,2,5],'edgecolor','w','legend',true)
%
%    <COPYRIGHT>
  p = inputParser;
  p.KeepUnmatched=true; 
  %p.PartialMatching=false;
  p.addParamValue('d',self.d,@isscalar);
  p.addParamValue('labels',[]); % select labels to draw
  p.addParamValue('labName',[]); % function for string legend label name
  p.addParamValue('legend',false,@islogical);
  p.addParamValue('mouse',true,@islogical);
  p.addParamValue('bounds',false,@islogical);
  p.addParamValue('color',[]); % default colors
  p.parse(varargin{:});
  R=p.Results;
  varargin=fc_tools.utils.deleteCellOptions(varargin,p.Parameters);
  if fc_tools.comp.isOctave()
    NotImplemented={'FaceAlpha','EdgeAlpha'};
    nv=length(varargin);
    varargin=fc_tools.utils.deleteCellOptions(varargin,NotImplemented);
    if length(varargin)<nv
      fprintf('[fc-simesh] ''FaceAlpha'' or ''EdgeAlpha'' not yet implemented in plotmesh\n');
    end
  end
  if isempty(R.labels), idxlab=self.find(R.d);else idxlab=self.find(R.d,R.labels);end
  if isempty(idxlab)
    if nargout>=1, varargout{1}=[];end
    if nargout==2, varargout{2}={};end
    return;
  end
  
  if isempty(R.labName), labName=funLabName(self,R.d); else labName=R.labName;end
  ColorOption={};
  if ~isempty(R.color), ColorOption={'color',fc_tools.graphics.colorstr2rgb(R.color)};R.legend=false;end
  PrevHold=fc_tools.graphics.SetHoldOn(true);
  h=zeros(length(idxlab),1);j=1;
  clegend=cell(length(idxlab),1);
  for ilab=[idxlab]
    if R.bounds 
      if (self.sTh{ilab}.d==2)
        Th=self.sTh{ilab}.getBoundMesh();
        h(j)=Th.plotmesh(ColorOption{:},varargin{:});
        clegend{j}=labName(self.sTh{ilab}.label);
        j=j+1;
      end
    else
      h(j)=self.sTh{ilab}.plotmesh(ColorOption{:},varargin{:});
      clegend{j}=labName(self.sTh{ilab}.label);
      j=j+1;
    end
  end
  if self.dim==3,  view(3);end
  if ((R.legend) && (j>1)), legend(h,clegend,'Location','NorthEastOutside');end
  if (R.mouse) && (~fc_tools.comp.isOctave())
    datacursormode on
    dcm = datacursormode(gcf);
    set(dcm,'UpdateFcn',@fc_simesh.showmeshinfo)
  end
  fc_tools.graphics.RestoreHold(PrevHold)
  if nargout>=1, varargout{1}=h;end
  if nargout==2, varargout{2}=clegend;end
end